﻿using System;
using System.Net;
using System.Drawing;
using TileServer;

public class SrtmMag : IServer
{
    protected string ImageTypeValue;
    protected int MaximumZoomValue;
    protected string ServerNameValue;
    protected string URL;

    public SrtmMag()
    {
        this.BaseImageType = ".png";
        this.BaseMaximumZoom = 21;
        this.BaseServerName = "SrtmMag";
        this.BaseURL = "http://wms.jpl.nasa.gov/wms.cgi?VERSION=1.1.1&REQUEST=GetMap&SRS=EPSG:4326&WIDTH=256&HEIGHT=256&LAYERS=srtm_mag&STYLES=default&TRANSPARENT=TRUE&FORMAT=image/png&BBOX=";
    }

    public bool DownloadTile(int X, int Y, int Zoom, string Filename)
    {
        bool flag;

		double WLON = LonFromXMerc(X, Zoom);
		double ELON = LonFromXMerc(X + 1, Zoom);
		double NLAT = LatFromYMerc(Y, Zoom);
		double SLAT = LatFromYMerc(Y+ 1, Zoom);	
				
		string _SLAT = Convert.ToString(SLAT);		
		string _WLON = Convert.ToString(WLON);
		string _NLAT = Convert.ToString(NLAT);
		string _ELON = Convert.ToString(ELON);
				
        string lBbox = _WLON + "," + _SLAT + "," + _ELON + "," + _NLAT; 
 
        string requestUriString = this.BaseURL + lBbox;        
        try
        {
            HttpWebRequest request = (HttpWebRequest) WebRequest.Create(requestUriString);
            HttpWebResponse response = (HttpWebResponse) request.GetResponse();
            if (response.ContentType.IndexOf("image") > -1)
            {
                Image.FromStream(response.GetResponseStream()).Save(Filename);
                flag = true;
            }
            else
            {
                flag = false;
            }
            response.Close();
        }
        catch 
        {
            flag = false;
        }
        return flag;
    }

    protected string BaseImageType
    {
        get
        {
            return this.ImageTypeValue;
        }
        set
        {
            this.ImageTypeValue = value;
        }
    }

    protected int BaseMaximumZoom
    {
        get
        {
            return this.MaximumZoomValue;
        }
        set
        {
            this.MaximumZoomValue = value;
        }
    }

    protected string BaseServerName
    {
        get
        {
            return this.ServerNameValue;
        }
        set
        {
            this.ServerNameValue = value;
        }
    }

    protected string BaseURL
    {
        get
        {
            return this.URL;
        }
        set
        {
            this.URL= value;
        }
    }

    public string ImageType
    {
        get
        {
            return this.ImageTypeValue;
        }
    }

    public int MaximumZoom
    {
        get
        {
            return this.MaximumZoomValue;
        }
    }

    public string ServerName
    {
        get
        {
            return this.ServerNameValue;
        }
    }
    
	public double LatFromYMerc(int Y, int Z)
	{
		double num = 3.14159265358979 / Math.Pow(2.0, (double) Z);
		double d = Y * num;
		d = 3.14159265358979 - d;
		d = Math.Exp(d);
		d = 2.0 * Math.Atan(d);
		return ((d - 1.570796326794895) / 0.017453292519943278);
	}

	public double LonFromXMerc(int X, int Z)
	{
		double num = 3.14159265358979 / Math.Pow(2.0, (double) Z);
		return (((X * num) - 3.14159265358979) / 0.017453292519943278);
	}
}

