Imports System.Net
Imports System.Drawing
Imports TileServer

Public Class SrtmMag
	Implements IServer
	Protected ImageTypeValue As String
	Protected MaximumZoomValue As Integer
	Protected ServerNameValue As String
	Protected URL As String

	Public Sub New()
		Me.BaseImageType = ".png"
		Me.BaseMaximumZoom = 21
		Me.BaseServerName = "SrtmMag"
		Me.BaseURL = "http://wms.jpl.nasa.gov/wms.cgi?VERSION=1.1.1&REQUEST=GetMap&SRS=EPSG:4326&WIDTH=256&HEIGHT=256&LAYERS=srtm_mag&STYLES=default&TRANSPARENT=TRUE&FORMAT=image/png&BBOX="
	End Sub

	Public Function DownloadTile(X As Integer, Y As Integer, Zoom As Integer, Filename As String) As Boolean Implements IServer.DownloadTile
		Dim flag As Boolean

		Dim WLON As Double = LonFromXMerc(X, Zoom)
		Dim ELON As Double = LonFromXMerc(X + 1, Zoom)
		Dim NLAT As Double = LatFromYMerc(Y, Zoom)
		Dim SLAT As Double = LatFromYMerc(Y + 1, Zoom)

		Dim _SLAT As String = Convert.ToString(SLAT)
		Dim _WLON As String = Convert.ToString(WLON)
		Dim _NLAT As String = Convert.ToString(NLAT)
		Dim _ELON As String = Convert.ToString(ELON)

		Dim lBbox As String = _WLON & "," & _SLAT & "," & _ELON & "," & _NLAT

		Dim requestUriString As String = Me.BaseURL & lBbox
		Try
			Dim request As HttpWebRequest = DirectCast(WebRequest.Create(requestUriString), HttpWebRequest)
			Dim response As HttpWebResponse = DirectCast(request.GetResponse(), HttpWebResponse)
			If response.ContentType.IndexOf("image") > -1 Then
				Image.FromStream(response.GetResponseStream()).Save(Filename)
				flag = True
			Else
				flag = False
			End If
			response.Close()
		Catch
			flag = False
		End Try
		Return flag
	End Function

	Protected Property BaseImageType() As String
		Get
			Return Me.ImageTypeValue
		End Get
		Set
			Me.ImageTypeValue = value
		End Set
	End Property

	Protected Property BaseMaximumZoom() As Integer
		Get
			Return Me.MaximumZoomValue
		End Get
		Set
			Me.MaximumZoomValue = value
		End Set
	End Property

	Protected Property BaseServerName() As String
		Get
			Return Me.ServerNameValue
		End Get
		Set
			Me.ServerNameValue = value
		End Set
	End Property

	Protected Property BaseURL() As String
		Get
			Return Me.URL
		End Get
		Set
			Me.URL = value
		End Set
	End Property

	Public ReadOnly Property ImageType() As String Implements IServer.ImageType
		Get
			Return Me.ImageTypeValue
		End Get
	End Property

	Public ReadOnly Property MaximumZoom() As Integer Implements IServer.MaximumZoom
		Get
			Return Me.MaximumZoomValue
		End Get
	End Property

	Public ReadOnly Property ServerName() As String Implements IServer.ServerName
		Get
			Return Me.ServerNameValue
		End Get
	End Property

	Public Function LatFromYMerc(Y As Integer, Z As Integer) As Double
		Dim num As Double = 3.14159265358979 / Math.Pow(2.0, CDbl(Z))
		Dim d As Double = Y * num
		d = 3.14159265358979 - d
		d = Math.Exp(d)
		d = 2.0 * Math.Atan(d)
		Return ((d - 1.5707963267949) / 0.0174532925199433)
	End Function

	Public Function LonFromXMerc(X As Integer, Z As Integer) As Double
		Dim num As Double = 3.14159265358979 / Math.Pow(2.0, CDbl(Z))
		Return (((X * num) - 3.14159265358979) / 0.0174532925199433)
	End Function
End Class

